################################################################################
# TITLE   Intro to R (StatLab Workshop, 2016 Spring)
# AUTHOR  Bommae Kim (bommaekim@email.virginia.edu)
# DATE    Feb-04-2016
################################################################################



# OUTLINE ----------------------------------------------------------------------

# RStudio interface

# Basics
# - Computation
# - Objects
# - Functions
# - Simple analysis examples
# - Help page

# Topics
# - Setting-up
# - Importing data
# - Data type
# - Object type
# - Indexing/subsetting data
# - Exporting data
# - Packages
# - Graph

# When you get errors

# Resources

# A complete example

#-------------------------------------------------------------------------------



# RStudio interface ------------------------------------------------------------

# Source (Script)
# Console
# Environment, History
# Files, Plots, Packages, Help, Viewer
### Tools >> Global Options >> Pane Layout

# Execute the current line
### ctrl+Enter (Windows) or cmd+Enter (Mac) in the script
### You can run multiple lines at once by highlighting them.

# Commenting
### R ignores anything after #. Comment your script as much as possible.

# Drop-down menu
### You might find some drop-down menus useful for you. They show shortcuts too.



# Computation ------------------------------------------------------------------

5 + 2
5-2   # Spacing doesn't matter
5   *   2
5 / 2
5^2

a <- 5   # Assign 5 to the object a
a

b = 2   # You could use '=' instead of '<-' to assign values to an object
b

a + b   # Same as using numbers
a-b
a   *   b
a / b
a^b

a
a <- 10000
a   # It's overwritten, but there are no warning messages!

sqrt(a)   # Do you know what this is for?
sqrt(10000)

log(a)
# Hmm, the result is not what I expected. I thought it would be 4 (from 10^4)

?log   # Let's check out the help page for the function log()
help(log)   # Does the same as ?log
### Help >>> [type in "log"]

# HELP
# - functionName{packageName}
# - Description
# - Usage: basic grammar, default values
# - Arguments: specification of the function (e.g., what type of data to use)
# - Details
# - Value
# - See Also: related functions  (i.e., check out 'sqrt' and 'Arithmetic')
# - Examples

log10(a)   # For base 10. Now I've got what I wanted



# Common errors

Log10(a)   # Why is this not working?
log10(A)   # How about this? Nope.
# R is case sensitive. Remember this!

log10(((a + 10000*5)^2)   # Watch your Console
# It doesn't return the results
# Something's wrong. Huh?
# First, move your cursor to Console, then escape (hit [esc] key on the left
# top)
      
)   # (Ignore this. Included this only to turn off annoying red boxes.)

# When we try something complicated, it often happens that we forget
# corresponding parentheses or brackets. Because R thinks your command is not
# completed, it is waiting for your next command. You could complete it by 
# typing in what is missing or you could simply ditch the current incomplete
# command by using the escape key. When you use the escape key, your cursor
# should be in Console, not in Source (the script).



# Mean of multiple numbers
5+1+5+4+5+3+3+1+2+4
(5+1+5+4+5+3+3+1+2+4)/10

# Here's an easier, lazier way: use object and function
x <- c(5,1,5,4,5,3,3,1,2,4)   # Assign multiple values to an object
x

sum(x)   # A function to sum values
length(x)   # Do you know what this does?
sum(x) / length(x)   # What about this? Or, you can simply...
mean(x)

# There are so many functions. How do I know which one to use?
# --> Google knows. Look it up!

results <- mean(x)   # Assign function results to an object
results

y <- c(2,3,7,8,6,2,0,0,3,1)
y

cor(x, y)   # A function could take multiple objects
plot(x, y)



# Functions can also take values other than objects. We will talk more about
# 'arguments' later.



# This is how things work in general in R. You assign 'things' to an object and
# manipulate objects using functions. R has specific rules for objects and
# functions. At the end of this workshop, you will be able to understand these
# specific rules.



# Setting-up -------------------------------------------------------------------

# Workspace

# What is workspace? There are things you've created so far in Environment. 
x

# Sometimes you may want to clean up your workspace (memory)
rm(list=ls())   # OR,
### Environment >>> [Broom icon]

x
y   # They're gone



# Working directory

# Where am I now?
getwd()

# This is my current working directory. If I save something while working in R,
# the file will be saved in this folder. Or, if I want to open a file, then the
# file should be in this folder (or, I should specify the full directory path
# where the file exists).

# Change your working directory
### Files >>> [navigate your folders] >>> More >>> Set As Working Directory

# See what message you've got in your console. You can set up your specific path
# using setwd(). You could copy & paste the line shown in your console to your
# script, so that you can use it as a part of your script.

# These are my directories. You should try your own.
setwd("~/Downloads")
getwd()
#setwd("/Users/bommae/Movies")
getwd()
#setwd("~/Google Drive/3_Consulting & Teaching/2016-1H Intro to R")
getwd()



# Loading/Importing Data -------------------------------------------------------

# Save your data in the current working directory, OR
# Set your working directory where you downloaded the data, OR
# Use the full path name in addition to the file name, OR
# Environment >>> Import Dataset >>> ...



# Let's look at your working directory on the right side. Do you see the file, 
# 'data_gradschool.csv'? If not, move the file into the current working
# directory, OR change your working directory to the directory where the file is
# located.

# If you can't find the downloaded data, run next:
# download.file("www.people.virginia.edu/~bk4kb/data_gradschool.txt", 'data_gradschool.txt')


# I strongly recommend using R script to import your data. Because we usually
# have multiple versions of dataset files, you won't remember which dataset you
# used in analysis after days or weeks.



read.table(data_gradschool.txt)   # What's wrong?
read.table('data_gradschool.txt')   # Does it look right?
?read.table  # HELP!

read.table('data_gradschool.txt', header=TRUE)  # Use arguments
df <- read.table("data_gradschool.txt", header=TRUE)
# Store the data in your workspace
df

# By the way, you can use double quotes and single quotes interchangeably in R.



# Check imported data
head(df)
tail(df)
summary(df)

str(df)
# object type, # of observations (rows), # of columns (variables)
# column name, data type, first some examples



# It's extremely important to understand data type and object type in R. They're
# typically the source of getting errors when you use R functions. If you have
# no experience with programming language, this might be confusing at first.



# Data and object type ---------------------------------------------------------

# Data types: numeric, character, logical, factor (and more)



# Create a thing
num <- 1  # Numeric
num

numVector <- 2, 3, 4   # Why is this not working?
numVector <- c(2, 3, 4)   # Combine values into a vector
numVector

chr <- 'one'   # Character
chr
chrVector <- c('two', 'three', 'four')
chrVector
chrVector_num <- c('2', '3', '4')
chrVector_num

# Difference?
numVector  # Numeric
chrVector_num  # Character

# Numeric or character, why does it matter?
numVector
sum(numVector)
chrVector_num
sum(chrVector_num)  # Why is this not working? What is the error message saying?

as.numeric(chrVector_num)   # I could coerce it into numeric
chrVector_asNumeric <- as.numeric(chrVector_num)
chrVector_asNumeric
sum(chrVector_asNumeric)   # OR
sum(as.numeric(chrVector_num))   # We can combine multiple functions



# How about this?
as.factor(numVector)   # Coercing to factor
facVector <- as.factor(numVector)
facVector   # Looks like numeric, but...

numVector
sum(numVector)

facVector
sum(facVector)  # Why is this not working?

chrVector_num  # Different from characters

# What is factor?
# Factors are typically nominal values with a limited number of levels, such as
# gender, city, or dog/cat. Even if you code such nominal values as integer,
# 0 and 1, you can set them as factors. Factors work differently from numeric
# values in R for statistical analysis or plotting. Some functions specifically
# require you to use factors or numerics in certain arguments. Usually
# characters don't work in statistical analysis. You should convert characters
# into factors. (Usually R automatically takes care of it.) (Actual examples
# later.)

facVector_chr <- factor(c('two', 'three', 'four'))
facVector_chr

chrVector

str(numVector)
str(chrVector_num)
str(facVector)



# How about this?
lgc <- TRUE   # Logical (Note that this is without quotation marks)
lgc
lgcVector <- c(TRUE, FALSE, TRUE)
lgcVector

### What is logical?
1 == 1   # Does 1 equal to 1?
1 > 2   # Is 1 greater than 2?
'a' == 'b'

numVector
numVector == 3   # Does 2 equal to 3? Does 3 equal to 3? Does 4 equal to 3?

# Logical values are different from characters or factors
as.character(lgcVector)   # Coercing into characters
lgcVector_asCharacter <-  as.character(lgcVector)

lgcVector_asCharacter
lgcVector   # Cosmetically different

sum(lgcVector_asCharacter)  # Why is this not working?
sum(lgcVector)  # What's happening? Functionally different!

# Internally, numerical values are attached to logical values: 0 for FALSE and
# 1 for TRUE. Just like numerics, you can do math with logical values. Normally
# logical values are used to select certain cases that meet some criteria ("find
# subjects whose weight is higher than 150 lbs"), and/or count the number of
# selected cases ("how many TRUEs are there for this selection criteria?"). This
# is particularly useful in subsetting data. You'll use logical values as 
# results of some computation rather than inputs. We will take a closer look at
# some examples soon.



# Why is it important to know the differences in data types? R functions are
# very specific about data types. If you don't use correct data type, you will
# get errors, or even worse, incorrect results. Incorrect data types are one of
# the most common errors you get in R.



# EXAMPLE: data types matter

df
str(df)   # Now we know what they mean
summary(df)   # Some functions work differently according to their data types



# Let's do some analysis.

# Linear regression: happiness predicted by age
happiness_age <- lm(happiness ~ age, data=df)   
summary(happiness_age)

# Linear regression, happiness predicted by field
happiness_field <- lm(happiness ~ field, data=df)   
summary(happiness_field)

# The analysis results are different in accordance to the data types (numeric or
# factor).



# Indexing and subsetting ------------------------------------------------------

# Oftentimes, we want to analyze subsets of data. For example, average weight of
# women, or median income of university professors whose age is over 50. We will
# go over how to subset data in this section. But, before that, I'd like to talk
# about object type first.



# Object type: vector, dataframe, list, (and more)

df
vec <- df$happiness   # Extracting a column from a dataframe (details later)

df   # dataframe: structured
vec   # vector: no structure

# A dataframe is a structured set of values. It looks like a table in MS-Word or
# Excel. (Note that, in R, 'table' is a type of object: a contingency table that
# summarizes frequency of values.) A dataset is normally represented as a
# dataframe. When you load data files, the datasets are usually imported as
# dataframes.

df
vec
str(df)
str(vec)



# Indexing

vec[3]   # objectName[element#] for vector
df[1, 3]   # objectName[row#, col#] for dataframe
# (because unlike a vector, a dataframe has two dimensions)
df[3, 1]

df[, 1]   # [all rows, 1st column], returning all entries of first column
df[3, ]   # [3rd row, all columns], returning all entries of 3rd row

df[, 3]   # Indexing by column number
df[, 'age']   # Indexing by column name using brackets
df$age   # Indexing by column name using $
# $ allows you to access only one column

# You can index multiple columns or rows at a time

1:5   # Creates sequential numbers
# You can use a function to create a specific set of sequential numbers. Check
# out ?seq

df[1:5, ]   # Returns rows 1 through 5
df[, 1:3]
df[, c(1,3)]   # Returns 1st and 3rd columns, without 2nd column
df[, c('age','happiness')]
# Note that the orders of the columns are changed as specified

# I prefer indexing by column name. It's much easier to understand your script
# and results later.

df$field   # What's the "object type" and "data type" of this result?
df$field[1]   # Indexing by column name and element number
df[1, 'field']   # Same as above

# How to change values of specific cells
df$age[5]
df$age[5] <- 30   # Assigns a value to an object (to the specific cell)
df$age[5]
df$age



# Subsetting--there are many ways to do the same thing

select <- c(T, T, F, T, F, F, F, F, F, T, F, F, F, T, F)
select   # Logical values, TRUE or FALSE, can be written T or F.
# I'd recommend spelling them out for readability for later.

length(select)
length(vec)

vec
vec[select]   # What's going on here?

vec
select
vec[select]   # Subsetted a vector by using logical values

df[select,]   # Works for dataframe too

# Although you can manually create such indicators (a vector of logical values),
# practical usage in real data analysis would be:
df$age > 30
selectAgeOver30 <- df$age > 30
# Created a logical vector to select subjects whose age is over 30

df[selectAgeOver30,]   # Select only TRUEs
df[df$age > 30,]   # You can just type in the condition statement for subsetting
# Behind the scene, logical values are working here

# You can combine multiple conditions. For example, you may want to select only
# female subjects who never married and have a college degree in engineering or
# music. To do so, you need to understand how relational operators and logical
# operators work.



# Relational operators

?Comparison

# Note that '==' for equal, and '!=' for not equal. '=' is not an equal sign in
# R. '=' is for assignment, same as '<-'.

df
df[df$field == Engineering]   # Why is this not working?
df[df$field == 'Engineering']   # Why is this not working?
df[df$field == 'Engineering',]   # Don't forget quotation marks or comma!
# We want to select certain rows, which meet the condition
# df$field == 'Engineering'.

df[df$field != 'Engineering',]   # If you want to select all but Engineering

df$age[df$field != 'Engineering',]   # Why is this not working?
df$age   # This is a vector, not a dataframe. It has only one dimension.
df$age[df$field != 'Engineering']

# You should know which object type you're dealing with. Depending on the object
# type (vector, dataframe, or something else), R works differently for indexing
# and subsetting (and for many other things).



# Logical operators

?Logic

# & for AND, | for OR, ! for NOT
# Do you understand perfectly how these work? I often find students 
# misunderstand logical operations. Let's draw some diagrams and check what they
# mean. These are very useful to know in R (and many other stats softwares).

df[df$field == 'Engineering' & df$age > 30,]
(df$field == 'Engineering') & (df$age > 30)   # Behind the scene

df[df$field == 'Engineering' | df$age > 30,]
(df$field == 'Engineering') | (df$age > 30)   # Behind the scene

# How many subjects do we have?
df[df$field == 'Engineering' | df$age > 30,]   # You can count the rows, OR
nrow(df[df$field == 'Engineering' | df$age > 30,])
# You can use a function, nrow(), to count the rows, OR
df$field == 'Engineering' | df$age > 30
sum(df$field == 'Engineering' | df$age > 30)
# You can count the number of TRUEs by summing TRUEs using sum() (because TRUE
# has a value of 1, as we've shown)

# You can use logical values to count the number of cases that meet the
# specified condition. You might see this “trick” when you look at online R
# resources. Even if you don't use it that way, it's good to understand why and
# how it works, because many other R users use logical values in such a way.



# Another way to subset data is to use a function subset() or a package 'dplyr'.
# They'll work better if you want to do more complex indexing and subsetting
# (e.g., removing part of data that meets certain conditions). Check them out
# online.



# Functions and arguments ------------------------------------------------------

# Exporting data ---------------------------------------------------------------

# Export data to a file

?write.table
write.table(df, file='myModifiedData.txt')   # Well, let's just try it out
read.table('myModifiedData.txt')   # Read it back to make sure it is correct

# I want to write it in csv file though
write.csv(df, file='myModifiedData.csv')
read.csv('myModifiedData.csv')   # Read it back. What do you think?
?write.table   # What could be the reason for the redundant column, X?

# Functions have some input components you can change. It's called 'argument'.
# Some arguments are required, some are optional. For optional arguments, their
# values are predefined by default (by the developer of the function). For
# example, if you don't provide object x (df in our example), the function 
# write.csv() won't work.

write.csv()
write.csv(x=df)
# It works. No file produced though (because we didn't specify a filename).
write.csv(file='test.csv')   # x is required
write.csv(x=df, file='test.csv')
write.csv(x=df, file='test.csv', na='XXX')
# Let's change the default value of the argument na
read.csv('test.csv')

# You can change the order of arguments if you clearly state the argument names.
# You can omit argument names only if you exactly follow the orders of all
# arguments, as the help page shows.

write.csv(file='test1.csv', x=df)
# Switched their orders but stated the argument names
write.csv('test2.csv', df)   # Doesn't work
write.csv(df, 'test3.csv')   # Followed the order shown on the help page 

# People often omit argument names for the first one or two arguments that have
# no default values. For optional arguments (with default values), it's a
# convention to state argument names when you change them. Also, it helps to
# understand your script later.

write.csv(df, file='test4.csv', na='HAHAHA')
read.csv('test4.csv')

# write.table() has many arguments but you don't have to define every argument.
# Only x has no default. Typically, the first one or two arguments are objects
# you want to manipulate and of course you should provide them. Most other
# arguments may work fine with their defaults. Sometimes the default values
# might not be doing "what you think" though! It's always good to check Help
# when you use a function for the first time.

# Let's get back to the redundant column. What could be the reason for that?
# Which argument should we change?
?write.table

write.csv(df, file='myModifiedData.csv', row.names=FALSE)
# Logical value is used to turn on and off the switch of the argument
read.csv('myModifiedData.csv')   # Looks good



# Save R workspace

save.image(file='myWorkSpace.RData')
# Save the current state of your workspace
rm(list=ls())   # Clear the memory. Your environment is empty now.
load('myWorkSpace.RData')   # They're back

# Probably your work takes days or weeks and you wouldn't want to run your
# scripts from the beginning every time you restart R. If you save your
# workspace, then you can simply load the RData later and resume your work from
# there. Also, when you quit Rstudio, it asks if you want to save your
# workspace. In that way, Rstudio assigns a name simply '.RData'. You just need
# to load it next time when you open Rstudio.

# Just like any other software, R sometimes crashes. It's always a good practice
# to save your script often: ctrl+S.

# you can save objects selectively (see Help, ?save). When you deal with a large
# dataset, it'd be a good idea to save it as a RData image because loading and
# data cleaning could take a long time.

# If you have multiple projects to work on, "Rstudio Projects" might be useful
# for you. Google it.



# Packages ---------------------------------------------------------------------

describe(df)  # Working? If not, why not?

# Install and load packages

# Package is a set of related functions. To use packages, you need to install
# them first, and then load them in your workspace. Here, installing means
# downloading from online and saving them in your computer (where R is
# installed). Loading means call them out into your workspace to use right now.

install.packages('psych')  # Just once
library(psych)  # Whenever you restart your RStudio
describe(df)
?describe   # Note the package name shows 'psych'

# Unlike basic functions, you should load them into your workspace every time
# you use.

# You need to reinstall packages whenever you install a newer version of R.

# There are thousands of packages! Some of the most popular packages for general
# purposes would be:
### graph: ggplot2
### data manipulation: dplyr, tidyr

# For your specific data analysis purposes, google the analysis method you'd
# like to use. Almost always there is a nice package that perfectly serves your
# goal.



# Graph ------------------------------------------------------------------------

# Histogram

hist(df)  # Why is this not working?

?hist   # Let's look at what data type or object type x should be

hist(df$age)
hist(df$age, breaks=10)  # Use arguments
hist(df$age, breaks=3, main='Histogram of Age', xlab='Age')

# Save or copy your plots
### Plots >> Export >> Save Plot as Image... >>
### Plots >> Export >> Copy Plot to Clipboard... >>

pdf('myHistogram.pdf')   # If you want to use R command
hist(df$age)
dev.off()



# You wouldn't want to use R base plots for your publication though. You can
# make beautiful plots using a package 'ggplot2.' Take a look at some examples
# online. Simply google images of ggplot2.



# When you get errors... -------------------------------------------------------

# 0. Don't panic! (Even professional programmers get errors all the time.)
# 1. Check for typos. (R is case sensitive!)
# 2. Check corresponding parentheses and quotation marks.
# 3. Check data type. (See Help page for the function.)
# 4. Check if required packages are loaded.
# 5. Restart Rstudio.
# 6. Contact StatLab :-)



# Resources for R beginners ----------------------------------------------------

# Cookbook for R: www.cookbook-r.com
# Google is your friend.



# Congratulations! Now you've got basics of R!
### Rstudio environment, how to set up working directory
### Data types: numeric, characters, factors, logical, and more
### Object types: vector, dataframe, and more
### Indxing and subsetting
### Functions and arguments
### Importing/exporting data, loading/saving R workspace
### Packages
### Graphs

# Most importantly,
### How to read Help
### Troubleshooting with errors
### How to find information (Google!)



# Here is a complete example ---------------------------------------------------

# Let's restart Rstudio. Just for a fresh start.

# Set up your working directory

# Load packages you need
library(psych)

# Import data
dat <- read.csv('data_gradschool.csv')

# Inspect data
head(dat); tail(dat)   # ; runs multiple commands at once
str(dat)
summary(dat)

# Summary statistics
describe(dat)

# Visualization
plot(dat$happiness)
plot(dat$field)
plot(dat$happiness, dat$age)
# Same function works differently according to data types and objects

# Linear regression: happiness regressed on age
myModel <- lm(happiness ~ age, data=dat)
summary(myModel)

# I want to run the model without Engineering
dat[dat$field != 'Engineering',]
myModel_excludeEngineering <- lm(happiness ~ age, data=dat[dat$field != 'Engineering',])
summary(myModel_excludeEngineering)

# I want to build a model to predict happiness from age
myNewData_age <- c(20, 25, 30, 35, 40)   # My new input data for prediction

summary(myModel)
# prediction = intercept + coefficient * AGE
-6.8425 + 0.3982 * myNewData_age

# Better way to do it
myModel$coefficients['(Intercept)'] + myModel$coefficients['age'] * myNewData_age

# How to get that information though?
str(myModel)
myModel$coefficients
myModel$coefficients['(Intercept)']
myModel$coefficients['age']

# If you want to make your script even clearer to understand
myIntercept <- myModel$coefficients['(Intercept)']
mySlope <- myModel$coefficients['age']
myPrediction <- myIntercept + mySlope * myNewData_age
myPrediction



# By the way, analysis results are typically a list, which is another object
# type we haven't covered. A list can include anything. You can access each
# information using assigned names (typically shown with $ sign in str). Same
# function has the same structure.



# Why is this better than using numbers?
# You can use the same script even if you make changes in your data or analysis.
# In addition, it will make your script easy to understand later for your sake.
# Days later,you won't remember where those numbers came from. You could even
# store them in new objects with better names if you want to. Then it'll be even
# easier to understand.



# Naming convention
# Use uppercase and lowercase together for readability. Underscore_ greatly
# increases readability.

# Avoid naming objects same as existing functions. For example, because 'c'
# exists as a function, it's better to avoid naming an object as 'c'. How to
# know if it already exists or not? Just type it.
c
table
apple

# When you name objects, consider making them meaningful for you to understand
# later as well as for your colleagues.



# Linear regression: happiness regressed on age
myModel <- lm(happiness ~ age, data=dat)
summary(myModel)

# Linear regression: happiness regressed on field
myModel_field <- lm(happiness ~ field, data=dat)
summary(myModel_field)   # compare to age model
summary(myModel)

# Age is numeric, and field is factor. The analysis works differently according
# to data types. Even if field was coded as numbers, if the data type is factor,
# then the results don't change.

str(dat)
levels(dat$field)
levels(dat$field) <- c(1, 2, 3)
levels(dat$field)
str(dat)

myModel_field_releveld <- lm(happiness ~ field, data=dat)
summary(myModel_field_releveld)

# Still the same results. It doesn't matter how they look. But, if you coerce
# them to be numeric by using as.numeric(), the results will change.

dat$field_num <- as.numeric(dat$field)
str(dat)
myModel_field_num <- lm(happiness ~ field_num, data=dat)
summary(myModel_field_num)

# By the way, if you want to remove a specific column:
dat
dat$field_num <- NULL
dat



# There are so many cool things you can do in R.
# Check them out here: www.cookbook-r.com (Cookbook for R)

# Have fun with R! 

# END --------------------------------------------------------------------------



################################################################################
# UVa StatLab
# Website: data.library.virginia.edu/statlab
# Email: statlab@virginia.edu
################################################################################